package com.exner.tools.jkbikemechanicaldisasterprevention

import android.content.pm.PackageManager
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.activity.viewModels
import androidx.compose.foundation.isSystemInDarkTheme
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.consumeWindowInsets
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.imePadding
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Done
import androidx.compose.material.icons.filled.Home
import androidx.compose.material3.BottomAppBar
import androidx.compose.material3.BottomAppBarDefaults
import androidx.compose.material3.Button
import androidx.compose.material3.DrawerDefaults
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExtendedFloatingActionButton
import androidx.compose.material3.FloatingActionButtonDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.NavigationDrawerItem
import androidx.compose.material3.NavigationRail
import androidx.compose.material3.NavigationRailItem
import androidx.compose.material3.PermanentDrawerSheet
import androidx.compose.material3.PermanentNavigationDrawer
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.windowsizeclass.ExperimentalMaterial3WindowSizeClassApi
import androidx.compose.material3.windowsizeclass.calculateWindowSizeClass
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.LayoutDirection
import androidx.compose.ui.unit.dp
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.UserPreferencesManager
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.StravaLinkViewModel
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.helpers.NavigationStyle
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.theme.KJsBikeMaintenanceCheckerTheme
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.theme.Theme
import dagger.hilt.android.AndroidEntryPoint
import javax.inject.Inject

@AndroidEntryPoint
class StravaLinkActivity : ComponentActivity() {

    @Inject
    lateinit var userPreferencesManager: UserPreferencesManager

    private val viewModel: StravaLinkViewModel by viewModels()

    @OptIn(ExperimentalMaterial3WindowSizeClassApi::class, ExperimentalMaterial3Api::class)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        enableEdgeToEdge()
        setContent {
            // night mode has two possible triggers:
            // - device may be in night mode
            // - force night mode setting may be on
            val userTheme = viewModel.userSelectedTheme.collectAsStateWithLifecycle()

            // window size class
            val windowSizeClass = calculateWindowSizeClass(this)

            // Strava API credentials
            val stravaApiKey = userPreferencesManager.stravaApiKey().collectAsStateWithLifecycle(
                initialValue = null
            )
            val stravaApiSecret =
                userPreferencesManager.stravaApiSecret().collectAsStateWithLifecycle(
                    initialValue = null
                )

            // code and scope
            if (intent?.action == "android.intent.action.VIEW" && intent?.data?.host == "jkbike.net") {
                val code: String? = intent.data?.getQueryParameter("code")
                val scope: String? = intent.data?.getQueryParameter("scope")
                if (!code.isNullOrBlank() && !scope.isNullOrBlank() && !stravaApiKey.value.isNullOrBlank() && !stravaApiSecret.value.isNullOrBlank()) {
                    viewModel.authenticateWithCode(
                        code,
                        stravaApiKey.value!!,
                        stravaApiSecret.value!!
                    )
                }
            }

            val navigationStyle =
                NavigationStyle.getNavigationStyleForWidthSizeClass(windowSizeClass.widthSizeClass)

            KJsBikeMaintenanceCheckerTheme(
                darkTheme = userTheme.value == Theme.Dark || (userTheme.value == Theme.Auto && isSystemInDarkTheme())
            ) {
                when (navigationStyle) {
                    NavigationStyle.BOTTOM_BAR -> { // compact
                        Scaffold(
                            topBar = {
                                TopAppBar(
                                    title = { Text(text = stringResource(R.string.top_bar_title)) },
                                    navigationIcon = {
                                        finish()
                                    }
                                )
                            },
                            bottomBar = {
                                BottomAppBar(
                                    actions = {
                                        finish()
                                    },
                                    floatingActionButton = {
                                        ExtendedFloatingActionButton(
                                            text = {
                                                Text(stringResource(R.string.dialog_btn_text_done))
                                            },
                                            icon = {
                                                Icon(
                                                    imageVector = Icons.Default.Done,
                                                    contentDescription = stringResource(R.string.btn_desc_back)
                                                )
                                            },
                                            onClick = {
                                                finish()
                                            },
                                            containerColor = BottomAppBarDefaults.bottomAppBarFabColor,
                                            elevation = FloatingActionButtonDefaults.bottomAppBarFabElevation()
                                        )
                                    }
                                )
                            }
                        ) { innerPadding ->
                            val newPadding = PaddingValues.Absolute(
                                innerPadding.calculateLeftPadding(LayoutDirection.Ltr),
                                innerPadding.calculateTopPadding(),
                                innerPadding.calculateRightPadding(LayoutDirection.Ltr),
                                0.dp
                            )
                            Surface(
                                modifier = Modifier
                                    .padding(newPadding)
                                    .consumeWindowInsets(newPadding)
                            ) {
                                Column {
                                    Spacer(modifier = Modifier.weight(0.4f))
                                    Text(text = "Connected")
                                    Spacer(modifier = Modifier.size(8.dp))
                                    Button(onClick = {
                                        finish()
                                    }) {
                                        Text(text = "OK")
                                    }
                                }
                            }
                        }
                    }

                    NavigationStyle.LEFT_RAIL -> { // medium
                        Scaffold(
                            modifier = Modifier
                                .imePadding(),
                            topBar = {
                                TopAppBar(
                                    title = { Text(text = stringResource(R.string.top_bar_title_long)) },
                                    navigationIcon = {
                                        finish()
                                    }
                                )
                            }
                        ) { innerPadding ->
                            val newPadding = PaddingValues.Absolute(
                                innerPadding.calculateLeftPadding(LayoutDirection.Ltr),
                                innerPadding.calculateTopPadding(),
                                innerPadding.calculateRightPadding(LayoutDirection.Ltr),
                                0.dp
                            )
                            Surface(
                                modifier = Modifier
                                    .padding(newPadding)
                                    .consumeWindowInsets(newPadding)
                            ) {
                                Row(
                                    modifier = Modifier.fillMaxSize()
                                ) {
                                    NavigationRail(
                                        modifier = Modifier.fillMaxHeight()
                                    ) {
                                        NavigationRailItem(
                                            selected = false,
                                            onClick = {
                                                finish()
                                            },
                                            icon = {
                                                Icon(
                                                    imageVector = Icons.Default.Home,
                                                    contentDescription = stringResource(R.string.tab_home)
                                                )
                                            },
                                            label = {
                                                stringResource(R.string.tab_home)
                                            }
                                        )
                                        // add this page?
                                    }
                                    DefaultSpacer()
                                    Scaffold(
                                        content = { innerPadding ->
                                            Surface(
                                                modifier = Modifier
                                                    .fillMaxSize()
                                                    .padding(innerPadding)
                                                    .consumeWindowInsets(innerPadding)
                                                    .padding(8.dp)
                                            ) {
                                                Text(text = "OK")
                                            }
                                        },
                                        bottomBar = {
                                            BottomAppBar(
                                                containerColor = MaterialTheme.colorScheme.surface,
                                                actions = {
                                                    finish()
                                                },
                                                floatingActionButton = {
                                                    ExtendedFloatingActionButton(
                                                        onClick = {
                                                            val pm: PackageManager = packageManager
                                                            val intent =
                                                                pm.getLaunchIntentForPackage(
                                                                    packageName
                                                                )
                                                            startActivity(intent)
                                                        },
                                                        icon = {
                                                            Icon(
                                                                imageVector = Icons.AutoMirrored.Default.ArrowBack,
                                                                contentDescription = stringResource(
                                                                    R.string.btn_desc_back
                                                                )
                                                            )
                                                        },
                                                        text = {
                                                            Text(text = stringResource(R.string.dialog_btn_text_done))
                                                        }
                                                    )
                                                }
                                            )
                                        }
                                    )
                                }
                            }
                        }
                    }

                    NavigationStyle.LEFT_DRAWER -> { // wide
                        Scaffold(
                            modifier = Modifier
                                .imePadding(),
                            topBar = {
                                TopAppBar(
                                    title = { Text(text = stringResource(R.string.top_bar_title_long)) },
                                    navigationIcon = {
                                        finish()
                                    }
                                )
                            }
                        ) { innerPadding ->
                            val newPadding = PaddingValues.Absolute(
                                innerPadding.calculateLeftPadding(LayoutDirection.Ltr),
                                innerPadding.calculateTopPadding(),
                                innerPadding.calculateRightPadding(LayoutDirection.Ltr),
                                0.dp
                            )
                            Surface(
                                modifier = Modifier
                                    .padding(newPadding)
                                    .consumeWindowInsets(newPadding)
                            ) {
                                Box(
                                    modifier = Modifier
                                        .fillMaxSize()
                                ) {
                                    PermanentNavigationDrawer(
                                        modifier = Modifier,
                                        drawerContent = {
                                            PermanentDrawerSheet(
                                                modifier = Modifier.width(200.dp),
                                                drawerContainerColor = DrawerDefaults.standardContainerColor
                                            ) {
                                                Column {
                                                    NavigationDrawerItem(
                                                        selected = false,
                                                        onClick = {
                                                            val pm: PackageManager = packageManager
                                                            val intent =
                                                                pm.getLaunchIntentForPackage(
                                                                    packageName
                                                                )
                                                            startActivity(intent)
                                                        },
                                                        icon = {
                                                            Icon(
                                                                imageVector = Icons.Default.Home,
                                                                contentDescription = stringResource(
                                                                    R.string.tab_home
                                                                )
                                                            )
                                                        },
                                                        label = {
                                                            Text(stringResource(R.string.tab_home))
                                                        }
                                                    )
                                                    // add this page?
                                                }
                                            }
                                        }
                                    ) {
                                        Scaffold(
                                            content = { padding ->
                                                Surface(
                                                    modifier = Modifier
                                                        .fillMaxSize()
                                                        .padding(padding)
                                                        .consumeWindowInsets(padding)
                                                        .padding(8.dp)
                                                ) {
                                                    Text(text = "OK")
                                                }
                                            },
                                            bottomBar = {
                                                BottomAppBar(
                                                    containerColor = MaterialTheme.colorScheme.surface,
                                                    actions = {
                                                        finish()
                                                    },
                                                    floatingActionButton = {
                                                        ExtendedFloatingActionButton(
                                                            onClick = {
                                                                val pm: PackageManager =
                                                                    packageManager
                                                                val intent =
                                                                    pm.getLaunchIntentForPackage(
                                                                        packageName
                                                                    )
                                                                startActivity(intent)
                                                            },
                                                            icon = {
                                                                Icon(
                                                                    imageVector = Icons.AutoMirrored.Default.ArrowBack,
                                                                    contentDescription = stringResource(
                                                                        R.string.btn_desc_back
                                                                    )
                                                                )
                                                            },
                                                            text = {
                                                                Text(text = stringResource(R.string.dialog_btn_text_done))
                                                            }
                                                        )
                                                    }
                                                )
                                            }
                                        )
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}
