package com.exner.tools.jkbikemechanicaldisasterprevention.network.strava

import com.exner.tools.jkbikemechanicaldisasterprevention.json.adapters.InstantJsonAdapter
import com.exner.tools.jkbikemechanicaldisasterprevention.network.strava.models.StravaDetailedAthlete
import com.exner.tools.jkbikemechanicaldisasterprevention.network.strava.models.StravaSummaryActivity
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.StravaApiTokenStorage
import com.squareup.moshi.Moshi
import kotlinx.datetime.Clock
import kotlinx.datetime.DateTimeUnit
import kotlinx.datetime.TimeZone
import kotlinx.datetime.minus
import okhttp3.OkHttpClient
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import retrofit2.Retrofit
import retrofit2.converter.moshi.MoshiConverterFactory

// built based on https://medium.com/swlh/simplest-post-request-on-android-kotlin-using-retrofit-e0a9db81f11a

class StravaApiManager(
    val stravaApiTokenStorage: StravaApiTokenStorage
) {

    fun retrieveTokens(
        code: String,
        key: String,
        secret: String,
        onResult: (StravaOAuthRequestResponse?) -> Unit
    ) {
        val stravaRepository = getStravaApi()

        // prepare the request
        val oAuthRequestData = StravaOAuthRequestData(
            clientId = key,
            clientSecret = secret,
            code = code,
            grantType = "authorization_code"
        )
        stravaRepository.postOAuthTokenRequest(oAuthRequestData).enqueue(
            object : Callback<StravaOAuthRequestResponse> {
                override fun onFailure(call: Call<StravaOAuthRequestResponse>, t: Throwable) {
                    onResult(null)
                }

                override fun onResponse(
                    call: Call<StravaOAuthRequestResponse>,
                    response: Response<StravaOAuthRequestResponse>
                ) {
                    val requestResponse = response.body()
                    val refreshToken = requestResponse?.refreshToken
                    val accessToken = requestResponse?.accessToken
                    if (!accessToken.isNullOrEmpty()) {
                        stravaApiTokenStorage.saveStravaAccessToken(accessToken)
                    }
                    if (!refreshToken.isNullOrEmpty()) {
                        stravaApiTokenStorage.saveStravaRefreshToken(refreshToken)
                    }
                    val expiresAt = requestResponse?.expiresAt
                    if (expiresAt != null) {
                        stravaApiTokenStorage.saveStravaExpiresAt(expiresAt)
                    }
                    // hand back
                    onResult(requestResponse)
                }
            }
        )
    }

    fun refreshTokens(
        refreshToken: String,
        key: String,
        secret: String,
        onResult: (StravaOAuthRefreshTokenRequestResponse?) -> Unit
    ) {
        val stravaRepository = getStravaApi()

        val oAuthRefreshTokenRequestData = StravaOAuthRefreshTokenRequestData(
            clientId = key,
            clientSecret = secret,
            refreshToken = refreshToken,
            grantType = "refresh_token"
        )
        stravaRepository.postOAuthRefreshTokenRequest(oAuthRefreshTokenRequestData).enqueue(
            object : Callback<StravaOAuthRefreshTokenRequestResponse> {
                override fun onFailure(
                    call: Call<StravaOAuthRefreshTokenRequestResponse>,
                    t: Throwable
                ) {
                    onResult(null)
                }

                override fun onResponse(
                    call: Call<StravaOAuthRefreshTokenRequestResponse>,
                    response: Response<StravaOAuthRefreshTokenRequestResponse>
                ) {
                    val requestResponse = response.body()
                    val refreshToken = requestResponse?.refreshToken
                    val accessToken = requestResponse?.accessToken
                    if (!accessToken.isNullOrEmpty()) {
                        stravaApiTokenStorage.saveStravaAccessToken(accessToken)
                    }
                    if (!refreshToken.isNullOrEmpty()) {
                        stravaApiTokenStorage.saveStravaRefreshToken(refreshToken)
                    }
                    // hand back
                    onResult(requestResponse)
                }
            }
        )
    }

    fun retrieveLoggedInAthlete(onResult: (StravaDetailedAthlete?) -> Unit) {
        val stravaRepository = getStravaApi()

        stravaRepository.getLoggedInAthlete().enqueue(
            object : Callback<StravaDetailedAthlete> {
                override fun onFailure(call: Call<StravaDetailedAthlete>, t: Throwable) {
                    onResult(null)
                }

                override fun onResponse(
                    call: Call<StravaDetailedAthlete>,
                    response: Response<StravaDetailedAthlete>
                ) {
                    val requestResponse = response.body()
                    onResult(requestResponse)
                }
            }
        )
    }

    fun retrieveLoggedInAthleteActivities(onResult: (List<StravaSummaryActivity>) -> Unit) {
        val stravaRepository = getStravaApi()

        val now = Clock.System.now()
        val before = now.epochSeconds
        val after = now.minus(10, DateTimeUnit.WEEK, TimeZone.currentSystemDefault()).epochSeconds
        val page = 1
        val perPage = 30
        stravaRepository.getLoggedInAthleteActivities(
            before = before,
            after = after,
            page = page,
            perPage = perPage
        ).enqueue(
            object : Callback<List<StravaSummaryActivity>> {
                override fun onFailure(call: Call<List<StravaSummaryActivity>>, t: Throwable) {
                    onResult(emptyList())
                }

                override fun onResponse(
                    call: Call<List<StravaSummaryActivity>>,
                    response: Response<List<StravaSummaryActivity>>
                ) {
                    val requestResponse = response.body()
                    onResult(requestResponse ?: emptyList())
                }
            }
        )
    }

    private fun getStravaApi(): StravaApi {
        // prepare the API client
        val moshi: Moshi = Moshi.Builder()
            .add(InstantJsonAdapter())
            .build()
        val interceptor = StravaAuthInterceptor(stravaApiTokenStorage)
        val okHttpClient = OkHttpClient.Builder()
            .addInterceptor(interceptor)
            .build()
        val retrofit = Retrofit.Builder()
            .baseUrl("https://www.strava.com/")
            .client(okHttpClient)
            .addConverterFactory(MoshiConverterFactory.create(moshi))
            .build()
        val stravaRepository = retrofit.create(StravaApi::class.java)
        return stravaRepository
    }
}