package com.exner.tools.jkbikemechanicaldisasterprevention.preferences

import android.content.Context
import android.content.SharedPreferences
import android.util.Log
import androidx.core.content.edit
import com.exner.tools.jkbikemechanicaldisasterprevention.R
import dagger.hilt.android.qualifiers.ApplicationContext
import kotlinx.datetime.Clock
import kotlinx.datetime.Instant
import javax.inject.Inject
import javax.inject.Singleton

private const val TAG = "SATS"

@Singleton
class StravaApiTokenStorage @Inject constructor(@ApplicationContext context: Context) {

    private var prefs: SharedPreferences =
        context.getSharedPreferences(context.getString(R.string.app_name), Context.MODE_PRIVATE)

    companion object {
        const val STRAVA_API_KEY = "strava_api_key"
        const val STRAVA_API_SECRET = "strava_api_secret"
        const val STRAVA_ACCESS_TOKEN = "strava_access_token"
        const val STRAVA_REFRESH_TOKEN = "strava_refresh_token"
        const val STRAVA_EXPIRES_AT = "strava_expires_at"
    }

    fun resetApiCredentials() {
        Log.d(TAG, "resetting Strava credentials")
        prefs.edit {
            putString(STRAVA_API_KEY, null)
            putString(STRAVA_API_SECRET, null)
            putString(STRAVA_ACCESS_TOKEN, null)
            putString(STRAVA_REFRESH_TOKEN, null)
            putLong(STRAVA_EXPIRES_AT, 0L)
        }
    }

    fun fetchStravaApiKey(): String? {
        return prefs.getString(STRAVA_API_KEY, null)
    }

    fun fetchStravaApiSecret(): String? {
        return prefs.getString(STRAVA_API_SECRET, null)
    }

    fun saveStravaAccessToken(token: String) {
        prefs.edit {
            putString(STRAVA_ACCESS_TOKEN, token)
        }
    }

    fun fetchStravaAccessToken(): String? {
        return prefs.getString(STRAVA_ACCESS_TOKEN, null)
    }

    fun saveStravaRefreshToken(token: String) {
        prefs.edit {
            putString(STRAVA_REFRESH_TOKEN, token)
        }
    }

    fun fetchStravaRefreshToken(): String? {
        return prefs.getString(STRAVA_REFRESH_TOKEN, null)
    }

    fun saveStravaExpiresAt(expiresAt: Long) {
        prefs.edit {
            putLong(STRAVA_EXPIRES_AT, expiresAt)
        }
    }

    fun fetchStravaExpiresAt(): Long {
        return prefs.getLong(STRAVA_EXPIRES_AT, 0)
    }

    fun getStravaIsConnected(): Boolean {
        // check whether we have an API key and a valid (non-expired) token
        Log.d(TAG, "check: is Strava connected?")
        val key: String = prefs.getString(STRAVA_API_KEY, "") ?: ""
        if (key.isNotEmpty()) {
            Log.d(TAG, "  API_KEY check")
            // right, step 1 is good, user has added key
            val token: String = prefs.getString(STRAVA_ACCESS_TOKEN, "") ?: ""
            if (token.isNotEmpty()) {
                Log.d(TAG, "  ACCESS_TOKEN check")
                // step 2 is good, they have received a token
                val now = Clock.System.now()
                val expiryEpoch = prefs.getLong(STRAVA_EXPIRES_AT, 0L)
                val tokenExpiresInstant = Instant.fromEpochMilliseconds(expiryEpoch * 1000L)
                if (now < tokenExpiresInstant) {
                    Log.d(TAG, "  token is valid")
                    // step 3, token is still valid!
                    return true
                } else {
                    Log.d(TAG, "  token expired")
                }
            }
        }
        Log.d(TAG, "  -> no")
        return false
    }
}
