package com.exner.tools.jkbikemechanicaldisasterprevention.preferences

import android.content.Context
import androidx.datastore.preferences.core.booleanPreferencesKey
import androidx.datastore.preferences.core.edit
import androidx.datastore.preferences.core.emptyPreferences
import androidx.datastore.preferences.core.stringPreferencesKey
import androidx.datastore.preferences.preferencesDataStore
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike.DistanceMeasure
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.theme.Theme
import dagger.hilt.android.qualifiers.ApplicationContext
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.flow.map
import javax.inject.Inject
import javax.inject.Singleton

private val Context.dataStore by preferencesDataStore("preferences")

@Singleton
class UserPreferencesManager @Inject constructor(
    @ApplicationContext appContext: Context
) {
    private val userDataStorePreferences = appContext.dataStore

    fun theme(): Flow<Theme> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            Theme.valueOf(preferences[KEY_THEME] ?: Theme.Auto.name)
        }
    }

    suspend fun setTheme(newTheme: Theme) {
        userDataStorePreferences.edit { preferences ->
            preferences[KEY_THEME] = newTheme.name
        }
    }

    fun todoListsExpire(): Flow<Boolean> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            preferences[KEY_TODO_LISTS_EXPIRE] == true
        }
    }

    suspend fun setTodoListsExpire(newExpire: Boolean) {
        userDataStorePreferences.edit { preferences ->
            preferences[KEY_TODO_LISTS_EXPIRE] = newExpire
        }
    }

    fun distanceMeasure(): Flow<DistanceMeasure> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            DistanceMeasure.valueOf(preferences[KEY_SI_OR_WEIRDO] ?: DistanceMeasure.KM.toString())
        }
    }

    suspend fun firstDistanceMeasure(): DistanceMeasure {
        return (userDataStorePreferences.data.first()[KEY_SI_OR_WEIRDO]) as DistanceMeasure? ?: DistanceMeasure.KM
    }

    suspend fun setDistanceMeasure(newDistanceMeasure: DistanceMeasure) {
        userDataStorePreferences.edit { preferences ->
            preferences[KEY_SI_OR_WEIRDO] = newDistanceMeasure.toString()
        }
    }

    fun stravaApiKey(): Flow<String?> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            if (preferences[KEY_STRAVA_API_KEY] == "") {
                null
            } else {
                preferences[KEY_STRAVA_API_KEY]
            }
        }
    }

    suspend fun setStravaApiKey(key: String?) {
        userDataStorePreferences.edit { preferences ->
            preferences[KEY_STRAVA_API_KEY] = key ?: ""
        }
    }

    fun stravaApiSecret(): Flow<String?> {
        return userDataStorePreferences.data.catch {
            emit(emptyPreferences())
        }.map { preferences ->
            if (preferences[KEY_STRAVA_API_SECRET] == "") {
                null
            } else {
                preferences[KEY_STRAVA_API_SECRET]
            }
        }
    }

    suspend fun setStravaApiSecret(secret: String?) {
        userDataStorePreferences.edit { preferences ->
            preferences[KEY_STRAVA_API_SECRET] = secret ?: ""
        }
    }

    private companion object {
        val KEY_THEME = stringPreferencesKey(name = "preference_theme")
        val KEY_TODO_LISTS_EXPIRE = booleanPreferencesKey(name = "preference_todo_lists_expire")
        val KEY_SI_OR_WEIRDO = stringPreferencesKey(name = "preference_distances_use_km")
        val KEY_STRAVA_API_KEY = stringPreferencesKey(name = "preference_strava_api_key")
        val KEY_STRAVA_API_SECRET = stringPreferencesKey(name = "preference_strava_api_secret")
    }
}