package com.exner.tools.jkbikemechanicaldisasterprevention.ui

import android.util.Log
import androidx.lifecycle.ViewModel
import com.exner.tools.jkbikemechanicaldisasterprevention.database.KJsRepository
import com.exner.tools.jkbikemechanicaldisasterprevention.network.strava.StravaApiManager
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.StravaApiTokenStorage
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.datetime.Clock
import kotlinx.datetime.Instant
import javax.inject.Inject

private const val TAG = "GSVM"

@HiltViewModel
class KJsGlobalScaffoldViewModel @Inject constructor(
    repository: KJsRepository,
    private val stravaApiTokenStorage: StravaApiTokenStorage
) : ViewModel() {

    private val _destinationTitle: MutableStateFlow<String> = MutableStateFlow("")
    val destinationTitle: StateFlow<String> = _destinationTitle

    fun setDestinationTitle(newTitle: String) {
        _destinationTitle.value = newTitle
    }

    val numberOfBikes = repository.observerNumberOfBikes
    val numberOfNonRetiredComponents = repository.observeNumberOfNonRetiredComponents
    val numberOfRetiredComponents = repository.observeNumberOfRetiredComponents
    val numberOfActivities = repository.observeNumberOfActivities

    val stravaIsConnected = stravaApiTokenStorage.getStravaIsConnected()

    fun disconnectStrava() {
        stravaApiTokenStorage.resetApiCredentials()
    }

    init {
        Log.d(TAG, "init")

        // check whether STRAVA_ACCESS_TOKEN exists but is expired, and refresh if necessary
        val accessToken = stravaApiTokenStorage.fetchStravaAccessToken()
        if (!accessToken.isNullOrEmpty()) {
            val now = Clock.System.now()
            val accessTokenExpiryEpoch = stravaApiTokenStorage.fetchStravaExpiresAt()
            val accessTokenExpiry = Instant.fromEpochMilliseconds(accessTokenExpiryEpoch * 1000L)
            if (now < accessTokenExpiry) {
                Log.d(TAG, "access token is valid")
            } else {
                Log.d(TAG, "will try to refresh token...")
                val refreshToken = stravaApiTokenStorage.fetchStravaRefreshToken()
                val key = stravaApiTokenStorage.fetchStravaApiKey()
                val secret = stravaApiTokenStorage.fetchStravaApiSecret()
                if (!refreshToken.isNullOrEmpty() && !key.isNullOrEmpty() && !secret.isNullOrEmpty()) {
                    Log.d(TAG, "all necessary information available, refreshing...")
                    val stravaApiManager = StravaApiManager(stravaApiTokenStorage)
                    stravaApiManager.refreshTokens(
                        refreshToken = refreshToken,
                        key = key,
                        secret = secret
                    ) {
                        Log.d(TAG, "token refresh successful!")
                    }
                }
            }
        } else {
            Log.d(TAG, "no access token yet for Strava")
        }
    }
}