package com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike

import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.AddTask
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.windowsizeclass.WindowHeightSizeClass
import androidx.compose.material3.windowsizeclass.WindowSizeClass
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import com.exner.tools.jkbikemechanicaldisasterprevention.R
import com.exner.tools.jkbikemechanicaldisasterprevention.database.entities.Activity
import com.exner.tools.jkbikemechanicaldisasterprevention.database.views.ActivityWithBikeData
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsGlobalScaffoldViewModel
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsResponsiveNavigation
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.ComponentWearLevelDialog
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.FullWidthButton
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.TodoListItem
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.helpers.KJsAction
import com.ramcosta.composedestinations.annotation.Destination
import com.ramcosta.composedestinations.annotation.RootGraph
import com.ramcosta.composedestinations.generated.destinations.ActivityAddDestination
import com.ramcosta.composedestinations.generated.destinations.BikeAddDestination
import com.ramcosta.composedestinations.generated.destinations.HomeDestination
import com.ramcosta.composedestinations.generated.destinations.StravaApiCredentialsDestination
import com.ramcosta.composedestinations.navigation.DestinationsNavigator
import kotlinx.datetime.Clock

@OptIn(ExperimentalFoundationApi::class)
@Destination<RootGraph>(start = true)
@Composable
fun Home(
    homeViewModel: HomeViewModel = hiltViewModel(),
    kJsGlobalScaffoldViewModel: KJsGlobalScaffoldViewModel,
    destinationsNavigator: DestinationsNavigator,
    windowSizeClass: WindowSizeClass
) {
    kJsGlobalScaffoldViewModel.setDestinationTitle("") // Home doesn't have a title

    KJsResponsiveNavigation(
        HomeDestination,
        destinationsNavigator,
        windowSizeClass,
        myFloatingActionButton = KJsAction(
            enabled = true,
            imageVector = Icons.Default.AddTask,
            contentDescription = stringResource(R.string.btn_text_add_activity),
            onClick = {
                destinationsNavigator.navigate(ActivityAddDestination)
            }
        ),
        headline = ""
    ) {
        val activitiesByBikes: List<ActivityWithBikeData> by homeViewModel
            .observeActivityWithBikeData.collectAsStateWithLifecycle(
                initialValue = emptyList()
            )
        val bikes by homeViewModel.observeBikes.collectAsStateWithLifecycle(
            initialValue = emptyList()
        )

        var showDialog by remember { mutableStateOf(false) }

        val stravaApiKey by homeViewModel.stravaApiKey.collectAsStateWithLifecycle(
            initialValue = null
        )
        val stravaApiSecret by homeViewModel.stravaApiSecret.collectAsStateWithLifecycle(
            initialValue = null
        )
        val stravaApiCredentialsConfigured =
            !(stravaApiKey.isNullOrBlank() || stravaApiSecret.isNullOrBlank())

        Column(
            modifier = Modifier
                .fillMaxWidth()
        ) {
            Text(text = stringResource(R.string.welcome_to_kj_do_it_now))
            DefaultSpacer()
            if (activitiesByBikes.any { activityWithBikeData ->
                    activityWithBikeData.rideUid == null || activityWithBikeData.rideUid < 0
                }) {
                val filteredActivitiesByBikes: List<ActivityWithBikeData> =
                    activitiesByBikes.filter { activityWithBikeData ->
                        activityWithBikeData.rideUid == null || activityWithBikeData.rideUid < 0
                    }
                LazyColumn(
                    modifier = Modifier
                        .fillMaxWidth()
                ) {
                    stickyHeader {
                        if (windowSizeClass.heightSizeClass != WindowHeightSizeClass.Compact) {
                            Text(
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .background(MaterialTheme.colorScheme.primaryContainer)
                                    .padding(8.dp),
                                text = stringResource(R.string.hdr_todos)
                            )
                        }
                    }

                    items(
                        items = filteredActivitiesByBikes,
                        key = { it.activityUid }) { activityByBike ->
                        val activity = Activity(
                            title = activityByBike.activityTitle,
                            description = activityByBike.activityDescription,
                            isCompleted = activityByBike.activityIsCompleted,
                            rideUid = null,
                            isEBikeSpecific = activityByBike.isEBikeSpecific,
                            rideLevel = activityByBike.activityRideLevel,
                            createdInstant = activityByBike.activityCreatedInstant,
                            dueDate = activityByBike.activityDueDate,
                            bikeUid = activityByBike.bikeUid ?: 0,
                            doneInstant = activityByBike.activityDoneDateInstant,
                            componentUid = activityByBike.activityComponentUid,
                            uid = activityByBike.activityUid
                        )
                        TodoListItem(
                            activity = activityByBike,
                            destinationsNavigator = destinationsNavigator,
                            onCheckboxCallback = { result ->
                                homeViewModel.updateActivity(
                                    activity = activity.copy(
                                        isCompleted = result,
                                        doneInstant = if (result) {
                                            Clock.System.now()
                                        } else {
                                            null
                                        }
                                    )
                                )
                                // popup in case this has a component attached
                                if (activityByBike.activityComponentUid != null) {
                                    // update component
                                    homeViewModel.updateAttachedComponent(
                                        componentUid = activityByBike.activityComponentUid,
                                        bikeUid = activityByBike.bikeUid
                                    )
                                    if (result) {
                                        // show UI for component wearlevel
                                        showDialog = true
                                    }
                                }
                            },
                        )
                        if (showDialog) {
                            ComponentWearLevelDialog(
                                onDismissRequest = {
                                    showDialog = false
                                },
                                onConfirmation = { wearLevel ->
                                    showDialog = false
                                    // write stuff
                                    if (activityByBike.activityComponentUid != null) {
                                        homeViewModel.logComponentWearLevel(
                                            componentUid = activityByBike.activityComponentUid,
                                            wearLevel = wearLevel
                                        )
                                    }
                                },
                            )
                        }
                    }
                }
            } else if (bikes.isNotEmpty()) { // we have bikes, but no activities so far
                Text(text = stringResource(R.string.select_a_tab_below_to_see_activities_for_short_rides_days_out_or_bike_holidays))
                DefaultSpacer()
                Text(text = stringResource(R.string.you_can_also_add_activities_manually_and_they_will_show_up_on_this_screen))
                DefaultSpacer()
                FullWidthButton(onClick = {
                    destinationsNavigator.navigate(ActivityAddDestination)
                }) {
                    Text(text = stringResource(R.string.btn_text_add_activity))
                }
                DefaultSpacer()
            } else { // no bikes so far
                Text(text = stringResource(R.string.you_can_add_your_bike_s))
                DefaultSpacer()
                FullWidthButton(onClick = {
                    destinationsNavigator.navigate(BikeAddDestination)
                }) {
                    Text(text = stringResource(R.string.btn_text_add_bike))
                }

                if (stravaApiCredentialsConfigured) {
                    DefaultSpacer()
                    Text(text = stringResource(R.string.you_can_import_bikes_from_strava_too))
                    DefaultSpacer()
                    FullWidthButton(onClick = {
                        destinationsNavigator.navigate(StravaApiCredentialsDestination)
                    }) {
                        Text(text = stringResource(R.string.set_up_connection_with_strava))
                    }
                }
            }
        }
    }
}
