package com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike

import android.content.Intent
import android.util.Log
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Dataset
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.material3.windowsizeclass.WindowSizeClass
import androidx.compose.material3.windowsizeclass.WindowWidthSizeClass
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.core.net.toUri
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import com.exner.tools.jkbikemechanicaldisasterprevention.R
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsGlobalScaffoldViewModel
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsResponsiveNavigation
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultPasswordFieldWithSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultTextFieldWithSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.FullWidthButton
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.TextAndSwitch
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.TextAndTriStateToggle
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.TextAndTwoStateToggle
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.helpers.KJsAction
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.theme.Theme
import com.ramcosta.composedestinations.annotation.Destination
import com.ramcosta.composedestinations.annotation.RootGraph
import com.ramcosta.composedestinations.generated.destinations.ManageTemplateActivitiesDestination
import com.ramcosta.composedestinations.generated.destinations.SettingsDestination
import com.ramcosta.composedestinations.navigation.DestinationsNavigator

@Destination<RootGraph>
@Composable
fun Settings(
    settingsViewModel: SettingsViewModel = hiltViewModel(),
    kJsGlobalScaffoldViewModel: KJsGlobalScaffoldViewModel,
    destinationsNavigator: DestinationsNavigator,
    windowSizeClass: WindowSizeClass
) {
    kJsGlobalScaffoldViewModel.setDestinationTitle(stringResource(R.string.hdr_settings))

    KJsResponsiveNavigation(
        SettingsDestination,
        destinationsNavigator,
        windowSizeClass,
        myActions = listOf(
            KJsAction(
                imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                contentDescription = stringResource(R.string.btn_desc_back),
                onClick = {
                    destinationsNavigator.navigateUp()
                }
            )
        ),
        myFloatingActionButton = KJsAction(
            imageVector = Icons.Default.Dataset,
            contentDescription = stringResource(R.string.btn_text_manage_template_activities),
            onClick = {
                destinationsNavigator.navigate(ManageTemplateActivitiesDestination)
            }
        ),
        headline = stringResource(R.string.hdr_settings)
    ) {
        val userSelectedTheme by settingsViewModel.userSelectedTheme.collectAsStateWithLifecycle()
        val todoListsExpire by settingsViewModel.todoListsExpire.collectAsStateWithLifecycle()
        val distancesMeasure by settingsViewModel.distancesMetric.collectAsStateWithLifecycle()
        val stravaApiKey by settingsViewModel.stravaApiKey.collectAsStateWithLifecycle(
            initialValue = null
        )
        val stravaApiSecret by settingsViewModel.stravaApiSecret.collectAsStateWithLifecycle(
            initialValue = null
        )
        val stravaIsConfigured = !stravaApiKey.isNullOrBlank() && !stravaApiSecret.isNullOrBlank()

        val context = LocalContext.current

        Column(
            modifier = Modifier
                .verticalScroll(rememberScrollState())
        ) {
            Text(
                text = stringResource(R.string.title_section_ui_visuals),
                style = MaterialTheme.typography.titleMedium
            )
            TextAndTriStateToggle(
                text = stringResource(R.string.lbl_theme),
                currentTheme = userSelectedTheme,
                updateTheme = { it: Theme ->
                    settingsViewModel.updateUserSelectedTheme(it)
                }
            )
            DefaultSpacer()
            HorizontalDivider()
            DefaultSpacer()
            Text(
                text = stringResource(R.string.title_section_program_logic),
                style = MaterialTheme.typography.titleMedium
            )
            TextAndSwitch(
                text = stringResource(R.string.lbl_todo_lists_rides_expire_after_2_days),
                checked = todoListsExpire
            ) {
                settingsViewModel.updateTodoListsExpire(it)
            }
            DefaultSpacer()
            HorizontalDivider()
            DefaultSpacer()
            Text(
                text = stringResource(R.string.title_section_language),
                style = MaterialTheme.typography.titleMedium
            )
            DefaultSpacer()
            if (windowSizeClass.widthSizeClass != WindowWidthSizeClass.Expanded) {
                Text(
                    modifier = Modifier.padding(16.dp, 0.dp),
                    text = stringResource(R.string.lbl_get_template_activities_in_language)
                )
            }
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(8.dp, 0.dp),
                verticalAlignment = Alignment.CenterVertically,
            ) {
                if (windowSizeClass.widthSizeClass == WindowWidthSizeClass.Expanded) {
                    Text(
                        modifier = Modifier.padding(16.dp, 0.dp),
                        text = stringResource(R.string.lbl_get_template_activities_in_language)
                    )
                    Spacer(modifier = Modifier.weight(0.1f))
                }
                TextButton(onClick = {
                    settingsViewModel.replaceTemplateActivitiesWithNewLanguage("en")
                }) {
                    Text(text = "English")
                }
                TextButton(onClick = {
                    settingsViewModel.replaceTemplateActivitiesWithNewLanguage("fr")
                }) {
                    Text(text = "Français")
                }
                TextButton(onClick = {
                    settingsViewModel.replaceTemplateActivitiesWithNewLanguage("de")
                }) {
                    Text(text = "Deutsch")
                }
                TextButton(onClick = {
                    settingsViewModel.replaceTemplateActivitiesWithNewLanguage("de-rCH")
                }) {
                    Text(text = "Deutsch (Schweiz)")
                }
            }
            DefaultSpacer()
            TextAndTwoStateToggle(
                text = stringResource(R.string.lbl_distances_use_metric),
                currentTheme = distancesMeasure,
                updateTheme = { it: DistanceMeasure ->
                    settingsViewModel.updateDistancesUseKm(it)
                }
            )
            DefaultSpacer()
            HorizontalDivider()
            DefaultSpacer()
            Text(
                text = stringResource(R.string.title_section_strava_integration),
                style = MaterialTheme.typography.titleMedium
            )
            DefaultSpacer()
            if (kJsGlobalScaffoldViewModel.stravaIsConnected) {
                FullWidthButton(onClick = {
                    kJsGlobalScaffoldViewModel.disconnectStrava()
                }) {
                    Text(text = stringResource(R.string.disconnect_from_strava))
                }
            } else {
                Text(text = stringResource(R.string.strava_credentials))
                DefaultSpacer()
                DefaultTextFieldWithSpacer(
                    value = stravaApiKey ?: "",
                    label = stringResource(R.string.lbl_strava_api_key),
                    placeholder = stringResource(R.string.placehldr_strava_api_key),
                    onValueChange = {
                        settingsViewModel.setStravaApiKey(
                            it.ifEmpty {
                                null
                            }
                        )
                    },
                )
                DefaultPasswordFieldWithSpacer(
                    value = stravaApiSecret ?: "",
                    label = stringResource(R.string.lbl_strava_api_secret),
                    placeholder = stringResource(R.string.placehldr_strava_api_secret),
                    onValueChange = {
                        settingsViewModel.setStravaApiSecret(
                            it.ifEmpty {
                                null
                            }
                        )
                    }
                )
                if (stravaIsConfigured && !settingsViewModel.isConnected) {
                    DefaultSpacer()
                    FullWidthButton(onClick = {
                        val intentUri = "https://www.strava.com/oauth/mobile/authorize".toUri()
                            .buildUpon()
                            .appendQueryParameter("client_id", stravaApiKey)
                            .appendQueryParameter("redirect_uri", "https://jkbike.net")
                            .appendQueryParameter("response_type", "code")
                            .appendQueryParameter("approval_prompt", "auto")
                            .appendQueryParameter(
                                "scope",
                                "read,profile:read_all,activity:read_all"
                            )
                            .build()

                        val intent = Intent(Intent.ACTION_VIEW, intentUri)
                        Log.d("Settings", "Intent: $intent")
                        context.startActivity(intent)
                        Log.d("Settings", "startActivity called.")
                    }) {
                        Text(text = stringResource(R.string.connect_with_strava))
                    }
                }
            }
        }
    }
}