package com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike.bikes

import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material.icons.filled.Done
import androidx.compose.material3.windowsizeclass.WindowSizeClass
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.hilt.navigation.compose.hiltViewModel
import com.exner.tools.jkbikemechanicaldisasterprevention.R
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsGlobalScaffoldViewModel
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsResponsiveNavigation
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultDateSelectorNullableWithSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultDateSelectorWithSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultNumberFieldWithSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultTextFieldWithSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.TextAndSwitch
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.helpers.KJsAction
import com.ramcosta.composedestinations.annotation.Destination
import com.ramcosta.composedestinations.annotation.RootGraph
import com.ramcosta.composedestinations.generated.destinations.BikeDeleteDestination
import com.ramcosta.composedestinations.generated.destinations.BikeEditDestination
import com.ramcosta.composedestinations.generated.destinations.ManageBikesDestination
import com.ramcosta.composedestinations.navigation.DestinationsNavigator
import kotlinx.datetime.LocalDateTime
import kotlinx.datetime.LocalTime
import kotlinx.datetime.TimeZone
import kotlinx.datetime.toInstant

@Destination<RootGraph>
@Composable
fun BikeEdit(
    bikeUid: Long,
    kJsGlobalScaffoldViewModel: KJsGlobalScaffoldViewModel,
    destinationsNavigator: DestinationsNavigator,
    windowSizeClass: WindowSizeClass
) {
    kJsGlobalScaffoldViewModel.setDestinationTitle(stringResource(R.string.hdr_edit_a_bike))

    val bikeEditViewModel =
        hiltViewModel<BikeEditViewModel, BikeEditViewModel.BikeEditViewModelFactory> { factory ->
            factory.create(bikeUid = bikeUid)
        }

    var modified by remember { mutableStateOf(false) }

    KJsResponsiveNavigation(
        BikeEditDestination,
        destinationsNavigator,
        windowSizeClass,
        myActions = listOf(
            KJsAction(
                imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                contentDescription = stringResource(R.string.btn_text_cancel),
                onClick = {
                    destinationsNavigator.navigateUp()
                }
            ),
            KJsAction(
                imageVector = Icons.Default.Delete,
                contentDescription = stringResource(R.string.btn_text_delete),
                onClick = {
                    destinationsNavigator.navigate(
                        BikeDeleteDestination(bikeUid = bikeUid)
                    )
                }
            )
        ),
        myFloatingActionButton = KJsAction(
            imageVector = Icons.Default.Done,
            contentDescription = stringResource(R.string.btn_text_save),
            onClick = {
                bikeEditViewModel.commitBike()
                modified = false
                destinationsNavigator.popBackStack(
                    ManageBikesDestination, inclusive = false
                )
            },
            enabled = modified
        ),
        headline = stringResource(R.string.hdr_edit_a_bike)
    ) {
        val bike by bikeEditViewModel.bike.observeAsState()
        val buildDateInstant = bike?.let {
            LocalDateTime(it.buildDate, LocalTime(12, 0, 0)).toInstant(
                TimeZone.currentSystemDefault()
            )
        }
        var selectedBuildDate = buildDateInstant?.toEpochMilliseconds()
        val lastUsedDateInstant = bike?.let {
            it.lastUsedDate?.let { it1 ->
                LocalDateTime(it1, LocalTime(12, 0, 0)).toInstant(
                    TimeZone.currentSystemDefault()
                )
            }
        }
        var selectedLastUsedDate = lastUsedDateInstant?.toEpochMilliseconds()
        val retirementDateInstance = bike?.let {
            it.retirementDate?.let { it1 ->
                LocalDateTime(it1, LocalTime(12, 0, 0)).toInstant(TimeZone.currentSystemDefault())
            }
        }
        var retirementDate = retirementDateInstance?.toEpochMilliseconds()

        Column(
            modifier = Modifier
                .verticalScroll(rememberScrollState())
        ) {
            DefaultTextFieldWithSpacer(
                value = bike?.name ?: stringResource(R.string.placehldr_name),
                onValueChange = {
                    bikeEditViewModel.updateName(it)
                    modified = true
                },
                label = stringResource(R.string.lbl_bike_name),
            )
            TextAndSwitch(
                text = stringResource(R.string.lbl_is_an_ebike),
                checked = bike?.isElectric == true
            ) {
                bikeEditViewModel.updateIsElectric(it)
                modified = true
            }
            DefaultDateSelectorWithSpacer(
                selectedDate = selectedBuildDate,
                label = stringResource(R.string.lbl_build_date),
                placeholder = stringResource(R.string.placehldr_yyyy_mm_dd),
                onDateSelected = {
                    selectedBuildDate = it
                    if (it != null) {
                        bikeEditViewModel.updateBuildDate(it)
                        modified = true
                    }
                }
            )
            DefaultNumberFieldWithSpacer(
                value = (bike?.mileage ?: "").toString(),
                onValueChange = { value ->
                    bikeEditViewModel.updateMileage(value.toIntOrNull() ?: 0)
                    modified = true
                },
                label = stringResource(R.string.lbl_mileage),
            )
            DefaultDateSelectorNullableWithSpacer(
                selectedDate = selectedLastUsedDate,
                label = stringResource(R.string.lbl_last_used_date),
                placeholder = stringResource(R.string.placehldr_yyyy_mm_dd),
                onDateSelected = {
                    selectedLastUsedDate = it
                    bikeEditViewModel.updateLastUsedDate(it)
                    modified = true
                }
            )
            TextAndSwitch(
                text = stringResource(R.string.is_active),
                checked = bike?.isActive == true
            ) {
                bikeEditViewModel.updateIsActive(it)
                modified = true
            }
            DefaultDateSelectorNullableWithSpacer(
                selectedDate = retirementDate,
                label = stringResource(R.string.lbl_retirement_date),
                placeholder = stringResource(R.string.placehldr_yyyy_mm_dd),
                onDateSelected = {
                    retirementDate = it
                    bikeEditViewModel.updateRetirementDate(it)
                    modified = true
                }
            )
            DefaultTextFieldWithSpacer(
                value = bike?.stravaGearId ?: stringResource(R.string.placehldr_strava_gear_id),
                onValueChange = { value ->
                    bikeEditViewModel.updateStravaGearId(value)
                    modified = true
                },
                label = stringResource(R.string.lbl_strava_gear_id)
            )
        }
    }
}
