package com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike.bikes

import android.content.Intent
import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.combinedClickable
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.automirrored.filled.DirectionsBike
import androidx.compose.material.icons.filled.Add
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.material3.windowsizeclass.WindowSizeClass
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.unit.dp
import androidx.core.net.toUri
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import com.exner.tools.jkbikemechanicaldisasterprevention.R
import com.exner.tools.jkbikemechanicaldisasterprevention.database.entities.Bike
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsGlobalScaffoldViewModel
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsResponsiveNavigation
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.FullWidthButton
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.IconSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.TextAndSwitch
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.helpers.KJsAction
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike.DistanceMeasure
import com.ramcosta.composedestinations.annotation.Destination
import com.ramcosta.composedestinations.annotation.RootGraph
import com.ramcosta.composedestinations.generated.destinations.BikeAddDestination
import com.ramcosta.composedestinations.generated.destinations.BikeEditDestination
import com.ramcosta.composedestinations.generated.destinations.ManageBikesDestination
import com.ramcosta.composedestinations.generated.destinations.SyncWithStravaDestination
import com.ramcosta.composedestinations.navigation.DestinationsNavigator

@OptIn(ExperimentalFoundationApi::class)
@Destination<RootGraph>
@Composable
fun ManageBikes(
    manageBikesViewModel: ManageBikesViewModel = hiltViewModel(),
    kJsGlobalScaffoldViewModel: KJsGlobalScaffoldViewModel,
    destinationsNavigator: DestinationsNavigator,
    windowSizeClass: WindowSizeClass
) {
    kJsGlobalScaffoldViewModel.setDestinationTitle(stringResource(R.string.hdr_manage_bikes))

    KJsResponsiveNavigation(
        currentDestination = ManageBikesDestination,
        destinationsNavigator = destinationsNavigator,
        windowSizeClass = windowSizeClass,
        myActions = listOf(
            KJsAction(
                imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                contentDescription = stringResource(R.string.btn_desc_back),
                onClick = {
                    destinationsNavigator.navigateUp()
                }
            )
        ),
        myFloatingActionButton = KJsAction(
            imageVector = Icons.Default.Add,
            contentDescription = stringResource(R.string.btn_text_add_bike),
            onClick = {
                destinationsNavigator.navigate(BikeAddDestination)
            }
        ),
        headline = stringResource(R.string.hdr_manage_bikes)
    ) {
        val bikes: List<Bike> by manageBikesViewModel.bikes.collectAsStateWithLifecycle(
            initialValue = emptyList()
        )

        val allBikes: List<Bike> by manageBikesViewModel.allBikes.collectAsStateWithLifecycle(
            initialValue = emptyList()
        )

        var showAllBikes by remember { mutableStateOf(false) }

        val distanceMeasure by manageBikesViewModel.distanceMeasure.collectAsStateWithLifecycle(
            initialValue = DistanceMeasure.KM
        )

        val stravaApiKey by manageBikesViewModel.stravaApiKey.collectAsStateWithLifecycle()
        val stravaApiSecret by manageBikesViewModel.stravaApiSecret.collectAsStateWithLifecycle()
        val stravaIsConfigured = !stravaApiKey.isNullOrBlank() && !stravaApiSecret.isNullOrBlank()
        val stravaIsConnected = manageBikesViewModel.isConnected

        val context = LocalContext.current

        LazyColumn(
            modifier = Modifier
                .fillMaxWidth()
        ) {
            item {
                TextAndSwitch(
                    text = "Show inactive bikes",
                    checked = showAllBikes,
                    onCheckedChange = {
                        showAllBikes = it
                    }
                )
            }
            stickyHeader {
                Text(
                    modifier = Modifier
                        .fillMaxWidth()
                        .background(MaterialTheme.colorScheme.primaryContainer)
                        .padding(8.dp),
                    text = stringResource(R.string.hdr_bikes),
                    style = MaterialTheme.typography.bodyMedium,
                )
            }

            val bikesToShow = if (showAllBikes) {
                allBikes
            } else {
                bikes
            }
            items(bikesToShow, key = { "bike.${it.uid}" }) { bike ->
                Surface(
                    modifier = Modifier
                        .combinedClickable(
                            onClick = {
                                destinationsNavigator.navigate(BikeEditDestination(bike.uid))
                            },
                        )
                        .padding(4.dp),
                ) {
                    Row(
                        modifier = Modifier.fillMaxWidth(),
                    ) {
                        val textColor = if (bike.isActive) {
                            Color.Unspecified
                        } else {
                            MaterialTheme.colorScheme.tertiary
                        }
                        Icon(
                            imageVector = Icons.AutoMirrored.Filled.DirectionsBike,
                            contentDescription = stringResource(R.string.bike),
                        )
                        IconSpacer()
                        Text(
                            text = bike.name,
                            modifier = Modifier.weight(1f),
                            color = textColor
                        )
                        DefaultSpacer()
                        Text(
                            text = "${bike.mileage} ${distanceMeasure.metric}",
                            color = textColor
                        )
                        DefaultSpacer()
                        Text(
                            text = if (bike.isActive) {
                                if (bike.lastUsedDate != null) {
                                    bike.lastUsedDate.toString()
                                } else {
                                    stringResource(R.string.placehldr_not_yet_used)
                                }
                            } else {
                                stringResource(R.string.not_active)
                            },
                            color = textColor
                        )
                    }
                }
            }

            stickyHeader {
                Text(
                    modifier = Modifier
                        .fillMaxWidth()
                        .background(MaterialTheme.colorScheme.primaryContainer)
                        .padding(8.dp),
                    text = stringResource(R.string.strava),
                    style = MaterialTheme.typography.bodyMedium
                )
            }

            item {
                if (stravaIsConfigured && !stravaIsConnected) {
                    FullWidthButton(onClick = {
                        val intentUri = "https://www.strava.com/oauth/mobile/authorize".toUri()
                            .buildUpon()
                            .appendQueryParameter("client_id", stravaApiKey)
                            .appendQueryParameter("redirect_uri", "https://jkbike.net")
                            .appendQueryParameter("response_type", "code")
                            .appendQueryParameter("approval_prompt", "auto")
                            .appendQueryParameter("scope", "read,profile:read_all,activity:read_all")
                            .build()

                        val intent = Intent(Intent.ACTION_VIEW, intentUri)
                        context.startActivity(intent)
                    }) {
                        Text(text = stringResource(R.string.connect_with_strava))

                    }
                }
            }

            item {
                if (stravaIsConnected) {
                    FullWidthButton(onClick = {
                        destinationsNavigator.navigate(SyncWithStravaDestination)
                    }) {
                        Text(text = stringResource(R.string.dropdown_item_sync_with_strava))
                    }
                }
            }
        }
    }
}