package com.exner.tools.jkbikemechanicaldisasterprevention.ui.jkbike.components

import androidx.compose.foundation.ExperimentalFoundationApi
import androidx.compose.foundation.background
import androidx.compose.foundation.combinedClickable
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Add
import androidx.compose.material.icons.filled.Dataset
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.material3.windowsizeclass.WindowSizeClass
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import com.exner.tools.jkbikemechanicaldisasterprevention.R
import com.exner.tools.jkbikemechanicaldisasterprevention.database.entities.Bike
import com.exner.tools.jkbikemechanicaldisasterprevention.database.entities.Component
import com.exner.tools.jkbikemechanicaldisasterprevention.database.views.ShelvedComponents
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsGlobalScaffoldViewModel
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsResponsiveNavigation
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.IconSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.SingleChoiceSegmentedButtonWithText
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.helpers.KJsAction
import com.ramcosta.composedestinations.annotation.Destination
import com.ramcosta.composedestinations.annotation.RootGraph
import com.ramcosta.composedestinations.generated.destinations.ComponentAddDestination
import com.ramcosta.composedestinations.generated.destinations.ComponentEditDestination
import com.ramcosta.composedestinations.generated.destinations.ManageComponentsDestination
import com.ramcosta.composedestinations.navigation.DestinationsNavigator

@OptIn(ExperimentalFoundationApi::class)
@Destination<RootGraph>
@Composable
fun ManageComponents(
    manageComponentsViewModel: ManageComponentsViewModel = hiltViewModel(),
    kJsGlobalScaffoldViewModel: KJsGlobalScaffoldViewModel,
    destinationsNavigator: DestinationsNavigator,
    windowSizeClass: WindowSizeClass
) {
    kJsGlobalScaffoldViewModel.setDestinationTitle(stringResource(R.string.hdr_manage_components))

    KJsResponsiveNavigation(
        ManageComponentsDestination,
        destinationsNavigator,
        windowSizeClass,
        myActions = listOf(
            KJsAction(
                imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                contentDescription = stringResource(R.string.btn_desc_back),
                onClick = {
                    destinationsNavigator.navigateUp()
                }
            )
        ),
        myFloatingActionButton = KJsAction(
            imageVector = Icons.Default.Add,
            contentDescription = stringResource(R.string.btn_text_add_component),
            onClick = {
                destinationsNavigator.navigate(ComponentAddDestination)
            }
        ),
        headline = stringResource(R.string.hdr_manage_components)
    ) {
        val components: List<Component> by manageComponentsViewModel.components.collectAsStateWithLifecycle(
            initialValue = emptyList()
        )
        val retiredComponents by manageComponentsViewModel.retiredComponents.collectAsStateWithLifecycle(
            initialValue = emptyList()
        )
        val componentsOnShelf: List<ShelvedComponents> by manageComponentsViewModel.componentsOnShelf.collectAsStateWithLifecycle(
            initialValue = emptyList()
        )
        val bikes: List<Bike> by manageComponentsViewModel.bikes.collectAsStateWithLifecycle(
            initialValue = emptyList()
        )
        var currentBike: Bike? by rememberSaveable { mutableStateOf(null) }
        var componentType by rememberSaveable {
            mutableStateOf(
                if (components.isNotEmpty()) {
                    ComponentType.ON_BIKE
                } else if (componentsOnShelf.isNotEmpty()) {
                    ComponentType.ON_SHELF
                } else if (retiredComponents.isNotEmpty()) {
                    ComponentType.RETIRED
                } else {
                    ComponentType.ON_BIKE
                }
            )
        }

        Column(
            modifier = Modifier
                .fillMaxSize()
        ) {
            SingleChoiceSegmentedButtonWithText(
                componentType = componentType,
                initCurrentBike = currentBike,
                bikes = bikes,
                onComponentTypeSwitched = { cType ->
                    componentType = cType
                },
                onSelected = { bike ->
                    currentBike = bike
                    componentType = ComponentType.ON_BIKE
                }
            )
            DefaultSpacer()

            val adaptedComponentsList: List<Component> =
                if (componentType == ComponentType.ON_SHELF) {
                    componentsOnShelf.map { componentOnShelf ->
                        Component(
                            name = componentOnShelf.name,
                            description = componentOnShelf.description,
                            acquisitionDate = componentOnShelf.acquisitionDate,
                            firstUseDate = componentOnShelf.firstUseDate,
                            lastCheckDate = componentOnShelf.lastCheckDate,
                            lastCheckMileage = componentOnShelf.lastCheckMileage,
                            currentMileage = componentOnShelf.currentMileage,
                            bikeUid = componentOnShelf.bikeUid,
                            titleForAutomaticActivities = componentOnShelf.titleForAutomaticActivities,
                            wearLevel = componentOnShelf.wearLevel,
                            retirementDate = componentOnShelf.retirementDate,
                            retirementReason = componentOnShelf.retirementReason,
                            checkIntervalMiles = componentOnShelf.checkIntervalMiles,
                            checkIntervalDays = componentOnShelf.checkIntervalDays,
                            uid = componentOnShelf.uid,
                        )
                    }
                } else if (componentType == ComponentType.RETIRED) {
                    retiredComponents.map { retiredComponent ->
                        Component(
                            name = retiredComponent.name,
                            description = retiredComponent.description,
                            acquisitionDate = retiredComponent.acquisitionDate,
                            firstUseDate = retiredComponent.firstUseDate,
                            lastCheckDate = retiredComponent.lastCheckDate,
                            lastCheckMileage = retiredComponent.lastCheckMileage,
                            currentMileage = retiredComponent.currentMileage,
                            bikeUid = retiredComponent.bikeUid,
                            titleForAutomaticActivities = retiredComponent.titleForAutomaticActivities,
                            wearLevel = retiredComponent.wearLevel,
                            retirementDate = retiredComponent.retirementDate,
                            retirementReason = retiredComponent.retirementReason,
                            checkIntervalMiles = retiredComponent.checkIntervalMiles,
                            checkIntervalDays = retiredComponent.checkIntervalDays,
                            uid = retiredComponent.uid,
                        )
                    }
                } else {
                    components.filter { component ->
                        (currentBike == null && component.bikeUid != null) || (currentBike != null && currentBike!!.uid == component.bikeUid)
                    }
                }

            LazyColumn(
                modifier = Modifier
                    .fillMaxWidth()
                    .weight(0.7f)
            ) {
                stickyHeader {
                    Text(
                        modifier = Modifier
                            .fillMaxWidth()
                            .background(MaterialTheme.colorScheme.primaryContainer)
                            .padding(8.dp),
                        text = stringResource(R.string.hdr_components),
                        style = MaterialTheme.typography.bodyMedium,
                    )
                }

                items(adaptedComponentsList, key = { "component.${it.uid}" }) { component ->
                    Surface(
                        modifier = Modifier
                            .combinedClickable(
                                onClick = {
                                    destinationsNavigator.navigate(
                                        ComponentEditDestination(
                                            component.uid
                                        )
                                    )
                                },
                            )
                            .padding(4.dp),
                    ) {
                        Row(
                            modifier = Modifier.fillMaxWidth()
                        ) {
                            Icon(
                                imageVector = Icons.Default.Dataset,
                                contentDescription = stringResource(R.string.component),
                            )
                            IconSpacer()
                            Text(
                                text = component.name,
                                maxLines = 1,
                                overflow = TextOverflow.Ellipsis
                            )
                            DefaultSpacer()
                            Text(
                                text = component.description,
                                overflow = TextOverflow.Ellipsis,
                                maxLines = 1,
                            )
                        }
                    }
                }
            }
        }
    }
}