package com.exner.tools.jkbikemechanicaldisasterprevention.ui.strava

import android.widget.Toast
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.Download
import androidx.compose.material3.Button
import androidx.compose.material3.HorizontalDivider
import androidx.compose.material3.Icon
import androidx.compose.material3.Text
import androidx.compose.material3.windowsizeclass.WindowSizeClass
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.saveable.rememberSaveable
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.text.style.TextOverflow
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.lifecycle.compose.collectAsStateWithLifecycle
import com.exner.tools.jkbikemechanicaldisasterprevention.R
import com.exner.tools.jkbikemechanicaldisasterprevention.database.entities.Bike
import com.exner.tools.jkbikemechanicaldisasterprevention.database.entities.StravaSummaryGear
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsGlobalScaffoldViewModel
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.KJsResponsiveNavigation
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.BikeSelectorWithTextButtonAndAddBikeOption
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.DefaultSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.FullWidthButton
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.IconSpacer
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.customComponents.ImportBikeDataDialog
import com.exner.tools.jkbikemechanicaldisasterprevention.ui.helpers.KJsAction
import com.ramcosta.composedestinations.annotation.Destination
import com.ramcosta.composedestinations.annotation.RootGraph
import com.ramcosta.composedestinations.generated.destinations.HomeDestination
import com.ramcosta.composedestinations.generated.destinations.SyncWithStravaDestination
import com.ramcosta.composedestinations.navigation.DestinationsNavigator
import kotlinx.datetime.Instant
import kotlinx.datetime.TimeZone
import kotlinx.datetime.toLocalDateTime
import kotlin.math.roundToInt

@Destination<RootGraph>
@Composable
fun SyncWithStrava(
    syncWithStravaViewModel: SyncWithStravaViewModel = hiltViewModel(),
    kJsGlobalScaffoldViewModel: KJsGlobalScaffoldViewModel,
    destinationsNavigator: DestinationsNavigator,
    windowSizeClass: WindowSizeClass
) {
    kJsGlobalScaffoldViewModel.setDestinationTitle("Sync with Strava") // Home doesn't have a title

    val context = LocalContext.current

    val bikesOnStrava by syncWithStravaViewModel.observeStravaSummaryGear.collectAsStateWithLifecycle(
        initialValue = emptyList()
    )

    val bikes: List<Bike> by syncWithStravaViewModel.observeBikes.collectAsStateWithLifecycle(
        initialValue = emptyList()
    )

    val mapOfStravaBikeConnections by syncWithStravaViewModel.mapOfStravaBikeConnections.collectAsStateWithLifecycle(
        initialValue = emptyMap()
    )

    var newBikeDialogOpen: Boolean by rememberSaveable { mutableStateOf(false) }
    var newGear: StravaSummaryGear? by rememberSaveable { mutableStateOf(null) }
    var newBike: Bike? by rememberSaveable { mutableStateOf(null) }


    KJsResponsiveNavigation(
        SyncWithStravaDestination,
        destinationsNavigator,
        windowSizeClass,
        myActions = listOf(
            KJsAction(
                imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                contentDescription = stringResource(R.string.btn_desc_back),
                onClick = {
                    destinationsNavigator.popBackStack(
                        HomeDestination,
                        inclusive = false
                    )
                }
            )
        ),
        myFloatingActionButton = KJsAction(
            enabled = syncWithStravaViewModel.isConnected,
            imageVector = Icons.Default.Download,
            contentDescription = stringResource(R.string.btn_text_update_distances_from_strava),
            onClick = {
                syncWithStravaViewModel.updateDistancesFromStrava(
                    onResult = {
                        Toast.makeText(
                            context,
                            context.getString(R.string.toast_updated_mileages),
                            Toast.LENGTH_LONG
                        ).show()
                    }
                )
            }
        ),
        headline = ""
    ) {
        Column(
            modifier = Modifier
                .verticalScroll(rememberScrollState())
        ) {
            if (syncWithStravaViewModel.isConnected) {
                if (bikesOnStrava.isNotEmpty()) {
                    Text(text = stringResource(R.string.bikes_retrieved_from_strava) + ": ${bikesOnStrava.size}")
                    HorizontalDivider()
                    LazyColumn(
                        modifier = Modifier.height(256.dp)
                    ) {
                        item {
                            Row(
                                modifier = Modifier.fillMaxWidth(),
                                horizontalArrangement = Arrangement.SpaceBetween,
                                verticalAlignment = Alignment.CenterVertically
                            ) {
                                Text(text = stringResource(R.string.strava))
                                Text(text = stringResource(R.string.local_this_app))
                            }
                        }
                        items(items = bikesOnStrava, key = { it.id!! }) { gear ->
                            Row(
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .height(36.dp),
                                horizontalArrangement = Arrangement.SpaceBetween,
                                verticalAlignment = Alignment.CenterVertically
                            ) {
                                Text(
                                    text = "${gear.name} (${gear.id})",
                                    overflow = TextOverflow.Ellipsis
                                )
                                BikeSelectorWithTextButtonAndAddBikeOption(
                                    currentBike = mapOfStravaBikeConnections[gear.id],
                                    bikes = bikes,
                                    displayLabel = false,
                                    onBikeSelected = { bike ->
                                        if (bike != null && gear.id != null) {
                                            if (bike.name == context.getString(R.string.dropdown_menu_item_bike_import_from_strava)) {
                                                // a bike to be imported
                                                newGear = gear
                                                newBike = bike
                                                newBikeDialogOpen = true
                                            } else {
                                                // an existing bike being linked
                                                syncWithStravaViewModel.onBikeSelected(
                                                    gearId = gear.id,
                                                    bike = bike,
                                                )
                                            }
                                        }
                                    }
                                )
                            }
                        }
                    }
                    if (newBikeDialogOpen) {
                        ImportBikeDataDialog(
                            onDismissRequest = { },
                            onConfirmation = { isEbike, acquisitionDate ->
                                newBikeDialogOpen = false
                                if (newGear != null && newBike != null) {
                                    val importedBike = newBike!!.copy(
                                        name = newGear!!.name
                                            ?: context.getString(R.string.default_name_from_strava),
                                        mileage = ((newGear!!.distance
                                            ?: 0.0f) / 1000).roundToInt(),
                                        stravaGearId = newGear!!.id,
                                        isElectric = isEbike,
                                        buildDate = Instant.fromEpochMilliseconds(acquisitionDate)
                                            .toLocalDateTime(
                                                TimeZone.currentSystemDefault()
                                            ).date
                                    )
                                    syncWithStravaViewModel.onBikeInserted(
                                        gear = newGear!!,
                                        bike = importedBike,
                                    )
                                }
                            }
                        )
                    }
                    DefaultSpacer()
                    if (mapOfStravaBikeConnections.isNotEmpty()) {
                        HorizontalDivider()
                        DefaultSpacer()
                        Text(text = stringResource(R.string.retrieve_current_distance_for_the_bikes_from_strava))
                        DefaultSpacer()
                        FullWidthButton(onClick = {
                            syncWithStravaViewModel.updateDistancesFromStrava(
                                onResult = {
                                    Toast.makeText(
                                        context,
                                        context.getString(R.string.toast_updated_mileages),
                                        Toast.LENGTH_LONG
                                    ).show()
                                }
                            )
                        }) {
                            Icon(
                                imageVector = Icons.Default.Download,
                                contentDescription = stringResource(R.string.btn_text_retrieve_bikes_from_strava)
                            )
                            IconSpacer()
                            Text(text = stringResource(R.string.btn_text_update_distances_from_strava))
                        }
                        DefaultSpacer()
                        Text(text = stringResource(R.string.update_latest_use_dates_for_bikes_from_strava))
                        FullWidthButton(onClick = {
                            syncWithStravaViewModel.updateLastRideDataFromStrava(
                                onResult = {
                                    Toast.makeText(
                                        context,
                                        context.getString(R.string.toast_updated_last_use_data),
                                        Toast.LENGTH_LONG
                                    ).show()
                                }
                            )
                        }) {
                            Icon(
                                imageVector = Icons.Default.Download,
                                contentDescription = stringResource(R.string.btn_text_retrieve_bikes_from_strava)
                            )
                            IconSpacer()
                            Text(text = stringResource(R.string.btn_text_update_last_ride_data_from_strava))
                        }
                    }
                } else {
                    Button(onClick = {
                        syncWithStravaViewModel.retrieveBikesFromStrava()
                    }) {
                        Icon(
                            imageVector = Icons.Default.Download,
                            contentDescription = stringResource(R.string.btn_text_retrieve_bikes_from_strava)
                        )
                        IconSpacer()
                        Text(text = stringResource(R.string.btn_text_retrieve_bikes_from_strava))
                    }
                }
            } else {
                Text(text = stringResource(R.string.unknown_status))
            }
        }
    }
}