package com.exner.tools.jkbikemechanicaldisasterprevention.ui.strava

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.exner.tools.jkbikemechanicaldisasterprevention.database.KJsRepository
import com.exner.tools.jkbikemechanicaldisasterprevention.database.entities.Bike
import com.exner.tools.jkbikemechanicaldisasterprevention.database.entities.StravaSummaryGear
import com.exner.tools.jkbikemechanicaldisasterprevention.network.BikeAndInstant
import com.exner.tools.jkbikemechanicaldisasterprevention.network.strava.StravaApiManager
import com.exner.tools.jkbikemechanicaldisasterprevention.preferences.StravaApiTokenStorage
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.launch
import kotlinx.datetime.TimeZone
import kotlinx.datetime.toLocalDateTime
import javax.inject.Inject
import kotlin.math.roundToInt

@HiltViewModel
class SyncWithStravaViewModel @Inject constructor(
    val repository: KJsRepository,
    private val stravaApiTokenStorage: StravaApiTokenStorage
) : ViewModel() {

    private val _listOfBikesOnStrava: MutableStateFlow<List<StravaSummaryGear>> = MutableStateFlow(
        emptyList()
    )
    val listOfBikesOnStrava: StateFlow<List<StravaSummaryGear>> = _listOfBikesOnStrava

    val observeBikes = repository.observeBikes

    val observeStravaSummaryGear = repository.observeStravaSummaryGear

    private val _mapOfStravaBikeConnections: MutableStateFlow<Map<String, Bike>> = MutableStateFlow(
        emptyMap()
    )
    val mapOfStravaBikeConnections: StateFlow<Map<String, Bike>> = _mapOfStravaBikeConnections

    val isConnected = stravaApiTokenStorage.getStravaIsConnected()

    init {
        // we do not want to retrieve bikes from Strava every time, so...
        viewModelScope.launch {
            // build the list
            _listOfBikesOnStrava.value = repository.getAllStravaSummaryGear()
            // build the map
            val listOfMappedBikes = repository.getAllBikes().filter { bike ->
                !bike.stravaGearId.isNullOrEmpty()
            }
            val tempMapOfStravaBikeConnections: MutableMap<String, Bike> = mutableMapOf()
            listOfMappedBikes.forEach { bike ->
                if (!bike.stravaGearId.isNullOrEmpty()) {
                    tempMapOfStravaBikeConnections.put(bike.stravaGearId, bike)
                }
            }
            _mapOfStravaBikeConnections.value = tempMapOfStravaBikeConnections
        }
    }

    fun retrieveBikesFromStrava() {
        viewModelScope.launch {
            if (stravaApiTokenStorage.getStravaIsConnected()) {
                val stravaApiManager = StravaApiManager(stravaApiTokenStorage)
                stravaApiManager.retrieveLoggedInAthlete { stravaDetailedAthlete ->
                    if (stravaDetailedAthlete != null) {
                        val tempBikes = stravaDetailedAthlete.bikes
                        viewModelScope.launch {
                            // put them into a list, and find existing links
                            val templist = listOfBikesOnStrava.value.toMutableList()
                            tempBikes?.forEach { gear ->
                                templist.add(gear)
                                if (gear.id != null) {
                                    val bike = repository.getBikeForStravaGearId(gear.id)
                                    if (bike != null) {
                                        val tempStravaBikeConnections =
                                            mapOfStravaBikeConnections.value.toMutableMap()
                                        tempStravaBikeConnections[gear.id] = bike
                                        _mapOfStravaBikeConnections.value =
                                            tempStravaBikeConnections
                                    }
                                    // store or update
                                    val tempGear = repository.getStravaSummaryGearById(gear.id)
                                    if (tempGear != null) {
                                        repository.updateStravaSummaryGear(gear)
                                    } else {
                                        repository.insertStravaSummaryGear(gear)
                                    }
                                }
                            }
                            _listOfBikesOnStrava.value = templist
                        }
                    }
                }
            }
        }
    }

    fun onBikeSelected(gearId: String, bike: Bike) {
        // update map
        val tempMap = mapOfStravaBikeConnections.value.toMutableMap()
        tempMap.remove(gearId)
        tempMap[gearId] = bike
        _mapOfStravaBikeConnections.value = tempMap
        viewModelScope.launch {
            // update _other_ bikes to remove this gear_id
            val otherBike = repository.getBikeForStravaGearId(gearId)
            if (otherBike != null) {
                val changedBike = otherBike.copy(
                    stravaGearId = ""
                )
                repository.updateBike(changedBike)
            }
            // update bike with strava_gear_id
            val changedSelectedBike = bike.copy(
                stravaGearId = gearId,
            )
            repository.updateBike(changedSelectedBike)
        }
    }

    fun onBikeInserted(gear: StravaSummaryGear, bike: Bike) {
        val gearId = gear.id!!
        // update map
        val tempMap = mapOfStravaBikeConnections.value.toMutableMap()
        tempMap.remove(gearId)
        tempMap[gearId] = bike
        _mapOfStravaBikeConnections.value = tempMap
        viewModelScope.launch {
            // update _other_ bikes to remove this gear_id
            val otherBike = repository.getBikeForStravaGearId(gearId)
            if (otherBike != null) {
                val changedBike = otherBike.copy(
                    stravaGearId = ""
                )
                repository.updateBike(changedBike)
            }
            // update bike with strava_gear_id
            val changedSelectedBike = bike.copy(
                stravaGearId = gearId,
            )
            val bikeUid = repository.insertBike(changedSelectedBike)
            // update the map so we have the bikeUid
            // it is needed to retrieve last ride date
            val temp1 = mapOfStravaBikeConnections.value[gearId]
            if (temp1 != null) {
                val temp2 = temp1.copy(
                    uid = bikeUid
                )
                val temp3 = mapOfStravaBikeConnections.value.toMutableMap()
                temp3[gearId] = temp2
                _mapOfStravaBikeConnections.value = temp3
            }
        }
    }

    fun updateDistancesFromStrava(onResult: () -> Unit) {
        viewModelScope.launch {
            listOfBikesOnStrava.value.forEach { gear ->
                if (mapOfStravaBikeConnections.value.containsKey(gear.id)) {
                    // we have this bike in both, Strava and the app
                    val bike = mapOfStravaBikeConnections.value[gear.id]
                    if (bike != null) {
                        val beforeMileage = bike.mileage
                        val newDistance = gear.distance
                        if (newDistance != null) {
                            val newRealDistance = (gear.distance / 1000).roundToInt()
                            if (beforeMileage < newRealDistance) {
                                val difference = newRealDistance - beforeMileage
                                // add this distance to every component attached to this bike
                                val allComponentsForBike =
                                    repository.getAllComponentsForBike(bike.uid)
                                allComponentsForBike.forEach { component ->
                                    val newMileage = difference + (component.currentMileage ?: 0)
                                    val updatedComponent = component.copy(
                                        currentMileage = newMileage
                                    )
                                    repository.updateComponent(updatedComponent)
                                }
                            }
                            // update the bike itself
                            val updatedBike = bike.copy(
                                mileage = newRealDistance
                            )
                            repository.updateBike(updatedBike)
                        }
                    }
                }
            }
            // done
            onResult()
        }
    }

    fun updateLastRideDataFromStrava(onResult: () -> Unit) {
        val mapOfGearBikeAndInstant: MutableMap<String, BikeAndInstant> = mutableMapOf()
        mapOfStravaBikeConnections.value.forEach { (gearId, bike) ->
            val bikeInstant = BikeAndInstant(bike = bike, instant = null)
            mapOfGearBikeAndInstant[gearId] = bikeInstant
        }
        viewModelScope.launch {
            if (stravaApiTokenStorage.getStravaIsConnected()) {
                val stravaApiManager = StravaApiManager(stravaApiTokenStorage)
                stravaApiManager.retrieveLoggedInAthleteActivities { listOfAthleteActivities ->
                    listOfAthleteActivities.forEach { summaryActivity ->
                        if (mapOfGearBikeAndInstant.containsKey(summaryActivity.gearId)) {
                            val bikeInstant = mapOfGearBikeAndInstant[summaryActivity.gearId]
                            if (bikeInstant != null && bikeInstant.instant == null) {
                                // haven't yet put a date on this bike! So do it!
                                val startDateLocal = summaryActivity.startDateLocal
                                if (startDateLocal != null) {
                                    mapOfGearBikeAndInstant.remove(summaryActivity.gearId)
                                    summaryActivity.gearId?.let {
                                        mapOfGearBikeAndInstant.put(
                                            it, bikeInstant.copy(
                                                instant = startDateLocal
                                            )
                                        )
                                    }
                                }
                            }
                        }
                    }
                    viewModelScope.launch {
                        // save update bikes
                        mapOfGearBikeAndInstant.forEach { (_, bikeAndInstant) ->
                            if (bikeAndInstant.instant != null) {
                                val bikeUid = bikeAndInstant.bike.uid
                                val bike = repository.getBikeByUid(bikeUid)
                                if (bike != null) {
                                    val updatedBike = bike.copy(
                                        lastUsedDate = bikeAndInstant.instant.toLocalDateTime(
                                            TimeZone.Companion.currentSystemDefault()
                                        ).date
                                    )
                                    repository.updateBike(updatedBike)
                                }
                            }
                        }
                    }
                    onResult()
                }
            }
        }
    }

}